@@ -561,7 +561,7 @@ XS_EXTERNAL(XS_heap_new_nv) {
     stash = gv_stashpvn("heap::nv", 8, GV_ADD);
     sv_bless(rv, stash);
 
-    ST(0) = rv;
+    ST(0) = sv_2mortal(rv);
     XSRETURN(1);
 }
 
@@ -1406,7 +1406,7 @@ XS_EXTERNAL(XS_heap_new) {
     stash = gv_stashpvn("heap", 4, GV_ADD);
     sv_bless(rv, stash);
 
-    ST(0) = rv;
+    ST(0) = sv_2mortal(rv);
     XSRETURN(1);
 }
 
